import { render, waitFor, screen } from '@testing-library/react';
import userEvent from '@testing-library/user-event';

import { PromQuery } from '@grafana/prometheus';
import { AnnoKeyCreatedBy } from 'app/features/apiserver/types';

import { QueryTemplatesList } from './QueryTemplatesList';
import { ListQueryTemplateApiResponse, QueryTemplate } from './api/endpoints.gen';


let data: ListQueryTemplateApiResponse = {
  items: [],
};

const mockQuery: PromQuery = {
  refId: 'A',
  datasource: {
    uid: 'Prometheus0',
    type: 'prometheus',
  },
  expr: 'go_gc_pauses_seconds_count{instance=\"host.docker.internal:3000\"}',
}

jest.mock('./api', () => {
  const actual = jest.requireActual('./api');
  return {
    ...actual,
    useDeleteQueryTemplateMutation: () => [() => {}],
    useListQueryTemplateQuery: () => {
      return {
        data: data,
        isLoading: false,
        error: null,
      };
    },
  };
});

jest.mock('./utils/dataFetching', () => {
  return {
    __esModule: true,
    useLoadQueryMetadata: () => {
      return {
        loading: false,
        value: [
          {
            index: '0',
            uid: '0',
            datasourceName: 'prometheus',
            datasourceRef: { type: 'prometheus', uid: 'Prometheus0' },
            datasourceType: 'prometheus',
            createdAtTimestamp: 0,
            query: mockQuery,
            queryText: 'http_requests_total{job="test"}',
            description: 'template0',
            user: {
              uid: 'viewer:JohnDoe',
              displayName: 'John Doe',
              avatarUrl: '',
            },
            error: undefined,
          },
        ],
      };
    },
    useLoadUsers: () => {
      return {
        value: {
          display: [
            {
              avatarUrl: '',
              displayName: 'john doe',
              identity: {
                name: 'JohnDoe',
                type: 'viewer',
              },
            },
          ],
        },
        loading: false,
        error: null,
      };
    },
  };
});

describe('QueryTemplatesList', () => {
  it('renders empty state', async () => {
    data = {};
    const mockOnSelectQuery = jest.fn();
    render(<QueryTemplatesList onSelectQuery={mockOnSelectQuery} />);
    await waitFor(() => {
      expect(screen.getByText(/You haven't saved any queries to your library yet/)).toBeInTheDocument();
    });
  });

  it('renders query', async () => {
    data.items = testItems;
    const mockOnSelectQuery = jest.fn();
    render(<QueryTemplatesList onSelectQuery={mockOnSelectQuery}/>);
    await waitFor(() => {
      // We don't really show query template title for some reason so creator name
      expect(screen.getByText(/John Doe/)).toBeInTheDocument();
    });
  });

  it('calls onSelectQuery when selecting the query', async () => {
    data.items = testItems;
    const mockOnSelectQuery = jest.fn();

    render(<QueryTemplatesList onSelectQuery={mockOnSelectQuery} />);
    expect(await screen.findByText(/John Doe/)).toBeInTheDocument();
    await userEvent.click(screen.getByText('Select query'));
    expect(mockOnSelectQuery).toHaveBeenCalledWith(mockQuery);
  });
});

const testItems: QueryTemplate[] = [
  {
    metadata: {
      name: 'TEST_QUERY',
      creationTimestamp: '2025-01-01T11:11:11.00Z',
      annotations: {
        [AnnoKeyCreatedBy]: 'viewer:JohnDoe',
      },
    },
    spec: {
      title: 'Test Query title',
      targets: [
        {
          variables: {},
          properties: {
            refId: 'A',
            datasource: {
              uid: 'Prometheus',
              type: 'prometheus',
            },
          },
        },
      ],
    },
  },
];
